// External Dependencies

// WordPress Dependencies
import { Component, Fragment } from '@wordpress/element';
import { __ } from '@wordpress/i18n';
import { PanelRow } from '@wordpress/components';
import { withInstanceId } from '@wordpress/compose';

// Internal Dependencies
import { Solid } from './control/solid';
import { Gradient } from './control/gradient';
import { Image } from './control/image';
import { deepMerge } from '../../../../../../utils';
import MediaQueryControlFlexRight from '../../../../media-query/control/flex-right';
import getData from '../../utils/get-data';
import getPropertyHasValueByMediaQueries from '../../utils/get-property-has-value-by-media-queries';

export const BACKGROUND_DEFAULT_VALUE = {
	type: 'solid',
	solid: {
		color: {},
	},
	gradient: {
		colors: [
			{ rgb: { r: 0, g: 0, b: 0, a: 1 } },
			{ rgb: { r: 255, g: 255, b: 255, a: 1 } },
		],
		type: 'linear',
		angle: undefined,
	},
	image: {
		horizontal: {},
		vertical: {},
	},
};

class Background extends Component {
	constructor() {
		super( ...arguments );

		this.setBackground = this.setBackground.bind( this );
	}

	getActiveData() {
		const { data, mediaActiveDevice, pseudoClass } = this.props;

		return getData(
			data,
			mediaActiveDevice,
			pseudoClass,
			{ background: BACKGROUND_DEFAULT_VALUE }
		);
	}

	renderControls( type ) {
		const data = this.getActiveData();
		const { background } = data;

		switch ( type ) {
			case 'gradient':
				return <Gradient background={ background } setBackground={ this.setBackground } />;
			case 'image':
				return <Image passThrough={ this.props.passThrough } background={ background } setBackground={ this.setBackground } />;
			default:
				return <Solid background={ background } setBackground={ this.setBackground } />;
		}
	}

	setBackground( newAttributes ) {
		const { setAttributes } = this.props;
		const data = this.getActiveData();

		const newBackground = deepMerge(
			data.background,
			newAttributes,
			{ arrayMerge: ( destinationArray, sourceArray ) => sourceArray }
		);

		if ( JSON.stringify( newBackground ) !== JSON.stringify( data.background ) ) {
			setAttributes( { background: newBackground } );
		}
	}

	render() {
		const { setAttributes, instanceId, mediaDevices } = this.props;
		const data = this.getActiveData();

		const options = [
			{ value: 'solid', label: __( 'Solid', 'wpv-views' ) },
			{ value: 'gradient', label: __( 'Gradient', 'wpv-views' ) },
			{ value: 'image', label: __( 'Image', 'wpv-views' ) },
		];

		const mediaUsages = getPropertyHasValueByMediaQueries(
			this.props.data,
			'background',
			mediaDevices,
			( newData ) => {
				if ( newData.type !== 'solid' ) {
					// Other than solid means active.
					return true;
				}

				// Check if a color iset.
				return !! newData.solid.color.rgb;
			}
		);

		return (
			<Fragment>
				<div className="components-base-control">
					<PanelRow className="tces-row-with-media-query-controls">
						<label htmlFor={ `tb-background-${ instanceId }` }>
							{ __( 'Type', 'wpv-views' ) }
						</label>
						<MediaQueryControlFlexRight mediaUsages={ mediaUsages } />
					</PanelRow>

					<select
						id={ `tb-background-${ instanceId }` }
						onChange={ e => setAttributes( { background: { ...data.background, type: e.target.value } } ) }
						value={ data.background.type }
						style={ { width: '100%' } }
					>
						{ options.map( ( item, index ) =>
							<option
								key={ `${ item.value }-${ index }` }
								value={ item.value }
							>
								{ item.label }
							</option>
						) }
					</select>
				</div>

				{ this.renderControls( data.background.type ) }

			</Fragment>
		);
	}
}

export default withInstanceId( Background );
