// WordPress dependencies.
import { Component, Fragment } from '@wordpress/element';
import { __ } from '@wordpress/i18n';
import { PanelRow } from '@wordpress/components';
import { BlockAlignmentToolbar } from '@wordpress/editor';
import { compose, createHigherOrderComponent } from '@wordpress/compose';
import { addFilter } from '@wordpress/hooks';
import { withSelect } from '@wordpress/data';

// Internal dependencies.
import MediaQueryControlFlexRight from '../../../media-query/control/flex-right';
import getData from '../utils/get-data';
import getPropertyHasValueByMediaQueries from '../utils/get-property-has-value-by-media-queries';
import { getKeyOrDefault } from '../../../../../utils/object-get-key-or-default';
import getCascadedAttributes from '../utils/get-cascaded-attributes';
import { name as storeMediaQuery } from '../../../media-query/store';

const VALID_CONTROLS = [ 'left', 'center', 'right', 'wide', 'full' ];
const DEFAULT_CONTROLS = [ 'left', 'center', 'right' ];
const WIDE_CONTROLS = [ 'wide', 'full' ];

// Backend apply of block align needs adjustments to the BlockListBlock component.
// Gutenberg offers an filter for this.
const applyBlockAlign = ( BlockListBlock ) => (
	( props ) => {
		const { attributes } = props;
		const style = attributes.style ? getCascadedAttributes( attributes.style ) : {};
		const blockAlign = style.blockAlign || false;

		let wrapperProps = props.wrapperProps;
		if ( blockAlign && VALID_CONTROLS.includes( blockAlign ) ) {
			wrapperProps = { ...wrapperProps, 'data-align': blockAlign };
		}

		return <BlockListBlock { ...props } wrapperProps={ wrapperProps } />;
	}
);

const withBlockAlign = createHigherOrderComponent(
	compose( [
		withSelect( ( select ) => {
			const { getMediaActiveDevice } = select( storeMediaQuery );
			return {
				mediaActiveDevice: getMediaActiveDevice(),
			};
		} ),
		applyBlockAlign,
	] )
);

addFilter( 'editor.BlockListBlock', 'toolset/common-es/block-align', withBlockAlign );

class BlockAlign extends Component {
	render() {
		const { setAttributes, mediaDevices, mediaActiveDevice, labels, pseudoClass } = this.props;

		const data = getData(
			this.props.data,
			mediaActiveDevice,
			pseudoClass
		);

		const mediaUsages = getPropertyHasValueByMediaQueries(
			this.props.data,
			'blockAlign',
			mediaDevices,
		);

		let controls = !! this.props.controls ? this.props.controls : DEFAULT_CONTROLS;

		controls = this.props.wideControlsEnabled ?
			controls :
			controls.filter( ( control ) => WIDE_CONTROLS.indexOf( control ) === -1 );

		if ( ! controls.length ) {
			return null;
		}

		return (
			<Fragment>
				<PanelRow className="tces-row-with-media-query-controls">
					{ /* eslint-disable-next-line */ }
					<label>{ getKeyOrDefault( labels, 'blockAlign', __( 'Block Alignment', 'wpv-views' ) ) }</label>
					<MediaQueryControlFlexRight mediaUsages={ mediaUsages } appliedOnceByCallingDevice={ true } />
				</PanelRow>

				<BlockAlignmentToolbar
					value={ data.blockAlign }
					onChange={ blockAlign => setAttributes( { blockAlign } ) }
					isCollapsed={ false }
					controls={ controls }
				/>
			</Fragment>
		);
	}
}

export default compose( [
	withSelect( ( select ) => {
		const { getSettings } = select( 'core/block-editor' );
		const settings = getSettings();
		return {
			wideControlsEnabled: settings.alignWide,
		};
	} ),
] )( BlockAlign );
