// WordPress dependencies
import { Fragment, Component } from '@wordpress/element';
import { ColorIndicator, PanelBody, TabPanel } from '@wordpress/components';
import { __ } from '@wordpress/i18n';
import { InspectorControls } from '@wordpress/editor';
import { merge } from 'lodash';
import { withSelect } from '@wordpress/data';
import { compose } from '@wordpress/compose';

// Internal dependencies
import { defaults } from './settings';
import CommonControls from './utils/common-controls';
import { getDifferenceOfObjects, colorAsString } from '../../../../utils';
import getData from './utils/get-data';
import getCascadedAttributes from './utils/get-cascaded-attributes';
import { MEDIA_QUERY_KEY_DESKTOP } from '../../media-query';
import { name as storeMediaQuery } from '../../media-query/store';

class Control extends Component {
	constructor() {
		super( ...arguments );

		this.setAttributes = this.setAttributes.bind( this );

		this.state = {
			pseudoclass: '',
		};
	}

	getDifferenceFromDefaults( data ) {
		if ( data === undefined ) {
			return {};
		}

		const defaultsBlock = this.props.defaults || {};
		const defaultsMerged = { ...defaults, ...defaultsBlock };
		data = { ...defaultsMerged, ...data };

		const diffFromDefaults = getDifferenceOfObjects( defaultsMerged, data );

		Object.keys( data ).forEach( key => {
			if ( ! defaults.hasOwnProperty( key ) ) {
				diffFromDefaults[ key ] = data[ key ];
			}
		} );

		return diffFromDefaults;
	}

	setAttributes( newData ) {
		const { setAttributes, data, storageKey, mediaActiveDevice } = this.props;
		const { pseudoclass } = this.state;

		const tabData = getData( data, mediaActiveDevice, pseudoclass );
		const newDataMergedWithPrevious = { ...tabData, ...newData };
		const style = mediaActiveDevice === MEDIA_QUERY_KEY_DESKTOP && ! pseudoclass ?
			this.getDifferenceFromDefaults( newDataMergedWithPrevious ) :
			newDataMergedWithPrevious;

		const key = storageKey || 'style';

		let newStyle = Object.assign( {}, data );

		if ( mediaActiveDevice !== MEDIA_QUERY_KEY_DESKTOP && pseudoclass ) {
			// Non-Desktop with a pseudoClass selected.
			newStyle[ mediaActiveDevice ] = newStyle[ mediaActiveDevice ] || {};
			newStyle[ mediaActiveDevice ][ pseudoclass ] = style;
		} else if ( mediaActiveDevice !== MEDIA_QUERY_KEY_DESKTOP ) {
			// Non-Desktop without a pseudoClass selected.
			newStyle[ mediaActiveDevice ] = style;
		} else if ( pseudoclass ) {
			// Desktop with a pseudoClass selected.
			newStyle[ pseudoclass ] = style;
		} else {
			// Desktop without a pseudoClass selected.
			newStyle = style;
		}

		// All other than Desktop without a pseudoClass selected.
		setAttributes( key === 'root' ? { ...newStyle } : { [ key ]: newStyle } );
	}

	render() {
		const {
			data,
			description,
			initialOpen,
			controls,
			labels,
			renderControlsOnly,
			renderPanelWithControlsOnly,
			passThrough,
			controlsMapping,
			preset,
			tabs,
			mediaActiveDevice,
		} = this.props;

		const styleWithDefaults = merge( {}, defaults, this.props.defaults || {}, data );
		const dataActive = getData( styleWithDefaults, mediaActiveDevice, this.state.pseudoclass );

		const presetTypography = preset === 'typography';
		const renderControls = ( ! Array.isArray( controls ) || controls.length === 0 ) && presetTypography ?
			[ 'fontFamily', 'fontSize', 'fontIconToolbar', 'lineHeight', 'letterSpacing', 'textColor', 'textTransform', 'textShadow' ] :
			controls;

		const currentMediaData = getCascadedAttributes( dataActive );

		const titleWithColorIndicators = (
			<Fragment>
				{ presetTypography ? __( 'Typography', 'wpv-views' ) : __( 'Style Settings', 'wpv-views' ) }
				<span>
					{ dataActive.textColor && renderControls.includes( 'textColor' ) &&
					<ColorIndicator
						key={ `title-text-color-${ this.state.pseudoclass }` }
						colorValue={ colorAsString( currentMediaData.textColor ) }
					/>
					}
					{ dataActive.backgroundColor && renderControls.includes( 'backgroundColor' ) &&
					<ColorIndicator
						key={ `title-background-color-${ this.state.pseudoclass }` }
						colorValue={ colorAsString( currentMediaData.backgroundColor ) }
					/>
					}
				</span>
			</Fragment>
		);

		const mainControl = ( tab ) => {
			return <CommonControls
				data={ styleWithDefaults || defaults }
				dataActive={ dataActive }
				controls={ renderControls }
				controlsMapping={ controlsMapping }
				setAttributes={ this.setAttributes }
				labels={ labels }
				passThrough={ passThrough }
				key={ `control-${ tab }` }
				pseudoClass={ this.state.pseudoclass }
			/>;
		};

		const panelWithMainControl = <PanelBody title={ titleWithColorIndicators } initialOpen={ initialOpen || false }>
			{ description &&
			<p>{ description }</p>
			}
			{ !! tabs &&
				<TabPanel className="tces-tabs"
					onSelect={ ( tabName ) => {
						const tab = tabs.find( item => item.name === tabName );
						if ( tab ) {
							this.setState( { pseudoclass: tab.pseudoclass } );
						}
					} }
					tabs={ tabs }
				>
					{ ( tab ) =>
						<Fragment key={ `tab-${ tab.name }` }>
							{ !! tab.description && <p className="tces-tabs__description">{ tab.description }</p> }
							<Fragment key={ `tab-content-${ tab.name }` }>
								{ mainControl( tab.name ) }
							</Fragment>
						</Fragment>
					}
				</TabPanel>
			}
			{ ! tabs &&
				mainControl( '' )
			}
		</PanelBody>;

		if ( renderControlsOnly ) {
			return mainControl();
		}

		if ( renderPanelWithControlsOnly ) {
			return panelWithMainControl;
		}

		return (
			<InspectorControls>
				{
					panelWithMainControl
				}
			</InspectorControls>
		);
	}
}

Control.defaultProps = {
	controls: [],
	passThrough: {},
	labels: {},
	renderControlsOnly: false,
	renderPanelWithControlsOnly: false,
	disableColorIdentificator: [],
	controlsMapping: {},
};

const ControlComposed = compose( [
	withSelect( ( select ) => {
		const { getMediaActiveDevice } = select( storeMediaQuery );
		return {
			mediaActiveDevice: getMediaActiveDevice(),
		};
	} ),
] )( Control );

export default ControlComposed;
