// External dependencies.
import { get } from 'lodash';

import { select } from '@wordpress/data';
import { name as storeMediaQuery } from '../../../media-query/store';
import { MEDIA_QUERY_KEY_DESKTOP } from '../../../media-query';
import getData from './get-data';
import { deepMerge } from '../../../../../utils';

// Some data needs to be removed from the current active data to have valid cascading.
// For example widthUnit should be removed when no width is there. Or undefined styles.
function filterCurrentActiveData( data ) {
	Object.keys( data ).forEach(
		key => {
			if (
				data[ key ] === undefined ||
				( key === 'widthUnit' && ! data.width )
			) {
				delete data[ key ];
			}
		}
	);

	return data;
}

// A custom search mechanic is required for some styles.
function customMerge( key ) {
	if ( key === 'padding' || key === 'margin' || key === 'boxShadow' ) {
		return ( parent, child ) => {
			return {
				...parent,
				...child,
				enabled: child.enabled || parent.enabled,
			};
		};
	} else if ( key === 'background' ) {
		return ( parent, child ) => {
			return child.type === 'solid' && ! get( child, 'solid.color.rgb' ) ?
				parent :
				child;
		};
	} else if ( key === 'display' ) {
		return ( parent, child ) => {
			return child;
		};
	}
}

/**
 * Returns an attributes set which fills undefined data with next higher device value.
 * This makes it possible to have the same styling like using media queries on the frontend.
 *
 * E.g. "Tablet" is active but has no color, but "Desktop" has. This function returns
 * the color of "Desktop" as it's also active for "Tablet" as long as there is no value for "Tablet".
 *
 * @param {Object} attributes
 * @return {*}
 */
export default function( attributes ) {
	const activeDevice = select( storeMediaQuery ).getMediaActiveDevice();

	if ( activeDevice === MEDIA_QUERY_KEY_DESKTOP ) {
		// Desktop is active. No need for cascading.
		return attributes;
	}

	// Get all devices.
	const devices = select( storeMediaQuery ).getMediaDevices();

	// Cascade the inline styles. Destkop > Tablet > Phone.
	let cascadedStyles = {};
	Object.keys( devices ).some( key => {
		if ( key === activeDevice ) {
			return true;
		}

		cascadedStyles = deepMerge( cascadedStyles, getData( attributes, key ), { customMerge } );
	} );

	// Get data of current active device.
	const activeDeviceData = filterCurrentActiveData( getData( attributes, activeDevice ) );

	// Current activeDevice has highest priority.
	attributes = deepMerge(
		cascadedStyles,
		activeDeviceData,
		{
			customMerge,
		}
	);

	return attributes;
}
