import { Component } from '@wordpress/element';
import { __ } from '@wordpress/i18n';
import { RangeControl, PanelBody, ToggleControl } from '@wordpress/components';

import { default as MediaQueryControlFlexRight } from '../media-query/control/flex-right';

class GridParameters extends Component {
	constructor( props ) {
		super( props );
		this.oldColumnsCount = null;
		this.columnChangeTimeout = null;
	}

	adjustSmallerScreens = ( newColumns ) => {
		if ( 'Desktop' === this.props.activeDevice ) {
			[ 'Tablet', 'Phone' ].forEach( device => {
				if ( this.props.attributes[ `columns${ device }` ].length > newColumns ) {
					const cWidth = 1 / newColumns,
						arr = [];
					for ( let i = 0; i < newColumns; i++ ) {
						arr.push( cWidth );
					}
					this.props.setAttributes( {
						[ `columns${ device }` ]: arr,
					} );
				}
			} );
		}
	};

	actualColumnCountChange = () => {
		const attr = `columns${ this.props.activeDevice }`,
			curCount = this.props.gridProps.columns.length;
		if ( curCount < this.oldColumnsCount ) {
			const { confirm } = window;
			if ( confirm( __( 'You\'re reducing grid size. Some data could be lost. Do you want to continue?', 'wpv-views' ) ) ) {
				this.props.setColumns( curCount );
			} else {
				const cWidth = 1 / this.oldColumnsCount,
					arr = [];
				for ( let i = 0; i < this.oldColumnsCount; i++ ) {
					arr.push( cWidth );
				}
				this.props.setAttributes( {
					[ attr ]: arr,
				} );
			}
		} else {
			this.props.setColumns( curCount );
		}
		this.adjustSmallerScreens( curCount );
		this.columnChangeTimeout = null;
		this.oldColumnsCount = null;
	};

	// We would like to notify user when we're reducing the columns amount in the grid
	// This could result in data loss, so we need to have a kind of debounce function
	// To show warning once user stop dragging the Column Count slider
	changeColumnsCount = ( value ) => {
		const cWidth = 1 / value,
			attr = `columns${ this.props.activeDevice }`,
			arr = [];
		for ( let i = 0; i < value; i++ ) {
			arr.push( cWidth );
		}

		if ( null === this.oldColumnsCount ) {
			this.oldColumnsCount = this.props.gridProps.columns.length;
		}
		if ( null !== this.columnChangeTimeout ) {
			clearTimeout( this.columnChangeTimeout );
		}
		this.props.setAttributes( {
			[ attr ]: arr,
		} );
		if ( this.props.skipColumnCountNotification || 'Desktop' !== this.props.activeDevice ) {
			this.columnChangeTimeout = null;
			this.oldColumnsCount = 0;
			this.adjustSmallerScreens( value );
		} else {
			this.columnChangeTimeout = setTimeout( () => this.actualColumnCountChange(), 500 );
		}
	};

	changeColumnGap = ( value ) => {
		this.props.setAttributes( {
			[ `columnGap${ this.props.activeDevice }` ]: value,
		} );
	};

	changeRowGap = ( value ) => {
		this.props.setAttributes( {
			[ `rowGap${ this.props.activeDevice }` ]: value,
		} );
	};

	render() {
		const { gridProps, toggleGapDisplay, getMediaUsages, maxColumnsCount, setColumnsReverse } = this.props;
		return (
			<PanelBody
				title={ __( 'Grid Settings', 'wpv-views' ) }
				initialOpen={ false }
			>
				<div className="tb-responsive-grid-parameter">
					<MediaQueryControlFlexRight mediaUsages={ getMediaUsages() } />
					<RangeControl
						label={ __( 'Columns Count', 'wpv-views' ) }
						value={ gridProps.columns.length }
						onChange={ this.changeColumnsCount }
						min={ 1 }
						max={ maxColumnsCount }
					/>
				</div>
				<div className="tb-responsive-grid-parameter">
					<MediaQueryControlFlexRight mediaUsages={ getMediaUsages() } />
					<RangeControl
						afterIcon={ 'px' }
						label={ __( 'Column Gap', 'wpv-views' ) }
						value={ gridProps.columnGap }
						onChange={ this.changeColumnGap }
						onMouseDown={ () => toggleGapDisplay( 'columnGapDisplay', true ) }
						onMouseUp={ () => toggleGapDisplay( 'columnGapDisplay', false ) }
						min={ 0 }
						max={ 200 }
					/>
				</div>
				<div className="tb-responsive-grid-parameter">
					<MediaQueryControlFlexRight mediaUsages={ getMediaUsages() } />
					<RangeControl
						label={ __( 'Row Gap', 'wpv-views' ) }
						value={ gridProps.rowGap }
						onChange={ this.changeRowGap }
						onMouseDown={ () => toggleGapDisplay( 'rowGapDisplay', true ) }
						onMouseUp={ () => toggleGapDisplay( 'rowGapDisplay', false ) }
						min={ 0 }
						max={ 200 }
					/>
				</div>
				{ ( ! this.props.hideColumnsReverseControl ) &&
					<div className="tb-responsive-grid-parameter">
						<MediaQueryControlFlexRight mediaUsages={ getMediaUsages() } />
						<ToggleControl
							checked={ gridProps.reverseColumns }
							label={ __( 'Reverse Columns', 'wpv-views' ) }
							onChange={ setColumnsReverse }
						/>
					</div>
				}
			</PanelBody>
		);
	}
}

export default GridParameters;
