// WordPress Dependencies
import { compose } from '@wordpress/compose';
import { withDispatch, withSelect } from '@wordpress/data';
import { Icon, Tooltip } from '@wordpress/components';
import { __ } from '@wordpress/i18n';

// Internal Dependencies
import { name as storeMediaQuery } from '../store';
import Abstract from './abstract';

const icon = <svg xmlns="http://www.w3.org/2000/svg" width="20px" height="20px" viewBox="0 0 303.311 303.311">
	<rect x="92.163" y="29.331" width="30.331" height="242.648" />
	<path d="M5.363,96.147l81.621,54.823L5.363,204.975V96.147z" />
	<rect x="180.632" y="29.331" width="30.331" height="242.648" />
	<path d="M297.763,204.975l-81.621-54.823l81.621-54.005V204.975z" />
</svg>;

class HeaderToolbar extends Abstract {
	constructor() {
		super( ...arguments );
		this.toggleUseMediaMaxWidthForEditor = this.toggleUseMediaMaxWidthForEditor.bind( this );

		this.state = {
			useMediaMaxWidthForEditor: true,
		};

		this.maximumWidthStyleId = 'tces-editor-maxium-width';
	}

	componentDidMount() {
		this.applyMediaMaxSizeToEditor();
	}

	componentDidUpdate() {
		this.applyMediaMaxSizeToEditor();
	}

	toggleUseMediaMaxWidthForEditor() {
		this.setState( { useMediaMaxWidthForEditor: ! this.state.useMediaMaxWidthForEditor } );
	}

	enableMaximumWidthForEditor() {
		const tagId = this.maximumWidthStyleId;
		const width = '100%';
		if ( ! document.getElementById( tagId ) ) {
			document.head.insertAdjacentHTML(
				'beforeend',
				`<style id="${ tagId }"> </style>` );
		}

		document.getElementById( tagId ).innerHTML = `
		body .wp-block { 
			max-width: ${ width } !important; 
			width: 100% !important;
		}
		body .editor-writing-flow {
			max-width: none !important;
			margin-left: auto !important;
			margin-right: auto !important;
		}
		body .editor-styles-wrapper .wp-block[data-align="full"] {
			left: 0 !important;
			width: calc( ${ width } + 90px ) !important;
			max-width: calc( ${ width } + 90px ) !important;
		}`;
	}

	disableMaximumWidthForEditor() {
		const styleTag = document.getElementById( this.maximumWidthStyleId );
		if ( styleTag ) {
			styleTag.innerHTML = '';
		}
	}

	applyMediaMaxSizeToEditor() {
		const { useMediaMaxWidthForEditor } = this.state;
		const DOMEditors = document.getElementsByClassName( 'block-editor-writing-flow' );

		if ( DOMEditors.length === 0 ) {
			// Core changed. Adjustment required.
			return;
		}

		const DOMEditor = DOMEditors[ 0 ];
		const removeProperties = () => {
			DOMEditor.style.removeProperty( 'max-width' );
			DOMEditor.style.removeProperty( 'margin-left' );
			DOMEditor.style.removeProperty( 'margin-right' );
		};

		if ( ! useMediaMaxWidthForEditor ) {
			// User do not want max width applied.
			removeProperties();
			this.disableMaximumWidthForEditor();
			return;
		}

		// User wants max width applied.
		const { mediaActiveDevice, mediaDevices } = this.props;
		const activeDevice = mediaDevices[ mediaActiveDevice ];
		// Desktop has no maxWidth, but for the editor use 1200 (px).
		const activeDeviceMaxWidth = activeDevice.maxWidth || 1200;

		if ( ! activeDeviceMaxWidth ) {
			// The current device has no max width (shouldn't happen).
			removeProperties();
			return;
		}

		// Allow editor to fully expand. Because most themes using the default editor max width. Which is very narrow.
		this.enableMaximumWidthForEditor();

		// Current device has a max width.
		DOMEditor.style.setProperty( 'max-width', activeDeviceMaxWidth + 60 + 'px', 'important' );
		DOMEditor.style.setProperty( 'margin-left', 'auto', 'important' );
		DOMEditor.style.setProperty( 'margin-right', 'auto', 'important' );
	}

	/* eslint-disable */
	render() {
		const tooltipText = this.state.useMediaMaxWidthForEditor ?
			__( "Ignore the device width in the preview.", 'wpv-views' ) :
			__( "Preview with the device width.", 'wpv-views' );

		return (
			<div className="tces-media-query-control tces-media-query-control-header-toolbar">
				<Tooltip text={ tooltipText } position={ 'bottom' }>
					<a className={ this.state.useMediaMaxWidthForEditor ? 'tces-media-device-active' : '' }
						onClick={ this.toggleUseMediaMaxWidthForEditor }><Icon
							size={ 20 } icon={ icon } /></a>
				</Tooltip> <span style={ { padding: '0 8px 0 10px' } }> | </span>
				{ this.getDevicesIcons( 'bottom' ) }
			</div>
		);
	}
	/* eslint-enable */
}

const HeaderToolbarComposed = compose( [
	withSelect( ( select ) => {
		const { getMediaDevices, getMediaActiveDevice } = select( storeMediaQuery );
		return {
			mediaDevices: getMediaDevices(),
			mediaActiveDevice: getMediaActiveDevice(),
		};
	} ),
	withDispatch( ( dispatch ) => {
		const { changeMediaDevice } = dispatch( storeMediaQuery );
		return {
			changeMediaDevice( targetDevice ) {
				changeMediaDevice( targetDevice );
			},
		};
	} ),
] )( HeaderToolbar );

export default HeaderToolbarComposed;
