/**
 * Padding and margin control
 *
 * @since 1.0.0
 */

// External dependencies
import PropTypes from 'prop-types';

// WordPress dependencies
import {
	Component,
	Fragment,
} from '@wordpress/element';
import { __ } from '@wordpress/i18n';
import {
	ToggleControl,
	PanelRow,
} from '@wordpress/components';

// Internal dependencies
import { deleteUndefinedNullInObject } from '../../../utils';
import './scss/edit.scss';

// Default unit, if no unit is used or an invalid unit.
const PADDING_MARGIN_DEFAULT_UNIT = 'px';

/**
 * Check if the control is used or not by passing values of margin OR padding.
 *
 * @param {Object} data
 * @return {boolean}
 */
export const PADDING_MARGIN_IN_USE = ( data ) => {
	return data.enabled === true;
};

export const PADDING_DEFAULT_VALUES = {
	enabled: false,
};

export const MARGIN_DEFAULT_VALUES = {
	enabled: false,
};

/**
 * Padding and margin control
 * Displays a box with margin/padding structure to set each element (top, botton, left, right)
 *
 * @since 1.0.0
 */
class PaddingMarginControl extends Component {
	onChange( attributes ) {
		if ( typeof this.props.onChange !== 'function' ) {
			// eslint-disable-next-line
			console.error( 'PaddingMarginControl.onChange must be a function.' );
			return;
		}

		this.props.onChange( deleteUndefinedNullInObject( attributes ) );
	}

	applyUnitToObjectValues( value ) {
		Object.keys( value ).forEach( key => {
			value[ key ] = value[ key ].replace( ' ', '' );
			const userInput = value[ key ];
			let userValue = userInput.match( /[+-]?\d+(?:\.\d+)?/g );

			if ( ! userValue ) {
				// No value at all.
				value[ key ] = null;
			} else if ( ! userInput.match( /\d+(?:\.\d+)?(em|ex|%|px|cm|mm|in|pt|pc|vh|vw|rem|ch|vmin|vmax)$/ ) ) {
				// No or invalid unit. Apply default unit.
				userValue = userInput.match( /[+-]?\d+(?:\.\d+)?/g );

				value[ key ] = userValue[ 0 ] + PADDING_MARGIN_DEFAULT_UNIT;
			}
		} );

		return value;
	}

	toggleChangedCallback( checked ) {
		if ( checked ) {
			this.onChange( {
				padding: { ...this.props.padding, enabled: true },
				margin: { ...this.props.margin, enabled: true },
			} );
		} else {
			this.onChange( {
				padding: { ...this.props.padding, enabled: false },
				margin: { ...this.props.margin, enabled: false },
			} );
		}
	}

	onChangeValueMargin( value ) {
		const attributes = {
			margin: { ...this.props.margin, ...value, enabled: true },
		};

		this.onChange( attributes );
	}

	onChangeValuePadding( value ) {
		const attributes = {
			padding: { ...this.props.padding, ...value, enabled: true },
		};

		this.onChange( attributes );
	}

	onBlurMargin( userInput ) {
		userInput = this.applyUnitToObjectValues( userInput );
		this.onChangeValueMargin( userInput );
	}

	onBlurPadding( userInput ) {
		userInput = this.applyUnitToObjectValues( userInput );
		this.onChangeValuePadding( userInput );
	}

	render() {
		const { label } = this.props;
		const rootClassName = 'wp-block-toolset-blocks-padding-margin';

		const dataPadding = Object.assign( {}, PADDING_DEFAULT_VALUES, this.props.padding );
		const dataMargin = Object.assign( {}, MARGIN_DEFAULT_VALUES, this.props.margin );
		const isEnabled = dataPadding.enabled || dataMargin.enabled;

		return (
			<Fragment>
				<PanelRow className="tces-row-with-media-query-controls">
					<ToggleControl
						label={ label }
						checked={ isEnabled }
						onChange={ checked => this.toggleChangedCallback( checked ) }
					/>
					{ this.props.mediaQueryControl }
				</PanelRow>

				{ isEnabled ?
					(
						<div className={ rootClassName }>
							<div className={ `${ rootClassName }__item1 ${ rootClassName }__cell` } data-label={ __( 'margin', 'wpv-views' ) }>
								<input
									type="text"
									value={ dataMargin.marginTop || '' }
									onChange={ event => this.onChangeValueMargin( { marginTop: event.target.value } ) }
									onBlur={ event => this.onBlurMargin( { marginTop: event.target.value } ) }
									placeholder="-"
								/>
							</div>
							<div className={ `${ rootClassName }__item2 ${ rootClassName }__cell` } data-label={ __( 'padding', 'wpv-views' ) }>
								<input type="text" value={ dataPadding.paddingTop || '' } onChange={ event => this.onChangeValuePadding( { paddingTop: event.target.value } ) } onBlur={ event => this.onBlurPadding( { paddingTop: event.target.value } ) } placeholder="-" />
							</div>
							<div className={ `${ rootClassName }__item3 ${ rootClassName }__cell` }>
								<input type="text" value={ dataMargin.marginLeft || '' } onChange={ event => this.onChangeValueMargin( { marginLeft: event.target.value } ) } onBlur={ event => this.onBlurMargin( { marginLeft: event.target.value } ) } placeholder="-"
								/>
							</div>
							<div className={ `${ rootClassName }__item4 ${ rootClassName }__cell` }>
								<input type="text" value={ dataPadding.paddingLeft || '' } onChange={ event => this.onChangeValuePadding( { paddingLeft: event.target.value } ) } onBlur={ event => this.onBlurPadding( { paddingLeft: event.target.value } ) } placeholder="-" />
							</div>
							<div className={ `${ rootClassName }__item5 ${ rootClassName }__cell` }>
								<input type="text" value={ dataPadding.paddingRight || '' } onChange={ event => this.onChangeValuePadding( { paddingRight: event.target.value } ) } onBlur={ event => this.onBlurPadding( { paddingRight: event.target.value } ) } placeholder="-" />
							</div>
							<div className={ `${ rootClassName }__item6 ${ rootClassName }__cell` }>
								<input type="text" value={ dataMargin.marginRight || '' } onChange={ event => this.onChangeValueMargin( { marginRight: event.target.value } ) } onBlur={ event => this.onBlurMargin( { marginRight: event.target.value } ) } placeholder="-" />
							</div>
							<div className={ `${ rootClassName }__item7 ${ rootClassName }__cell` }>
								<input type="text" value={ dataPadding.paddingBottom || '' } onChange={ event => this.onChangeValuePadding( { paddingBottom: event.target.value } ) } onBlur={ event => this.onBlurPadding( { paddingBottom: event.target.value } ) } placeholder="-" />
							</div>
							<div className={ `${ rootClassName }__item8 ${ rootClassName }__cell` }>
								<input type="text" value={ dataMargin.marginBottom || '' } onChange={ event => this.onChangeValueMargin( { marginBottom: event.target.value } ) } onBlur={ event => this.onBlurMargin( { marginBottom: event.target.value } ) } placeholder="-" />
							</div>
							<div className={ `${ rootClassName }__content ${ rootClassName }__cell` }></div>
						</div>
					) : (
						<Fragment />
					)
				}
			</Fragment>
		);
	}
}

PaddingMarginControl.defaultProps = {
	label: __( 'Margin / Padding', 'wpv-views' ),
	margin: MARGIN_DEFAULT_VALUES,
	padding: PADDING_DEFAULT_VALUES,
};

PaddingMarginControl.propTypes = {
	label: PropTypes.string,
	margin: PropTypes.shape( {
		enabled: PropTypes.bool,
		marginTop: PropTypes.string,
		marginLeft: PropTypes.string,
		marginRight: PropTypes.string,
		marginBottom: PropTypes.string,
	} ),
	padding: PropTypes.shape( {
		enabled: PropTypes.bool,
		paddingTop: PropTypes.string,
		paddingLeft: PropTypes.string,
		paddingRight: PropTypes.string,
		paddingBottom: PropTypes.string,
	} ),
	onChange: PropTypes.func.isRequired,
};

export default PaddingMarginControl;
