import { __, sprintf } from '@wordpress/i18n';
import { Component } from '@wordpress/element';
import Select from 'react-select';
import AsyncSelect from 'react-select/async';
import PropTypes from 'prop-types';
import { stringify } from 'querystringify';
import { debounce } from 'lodash';

class ReactSelect extends Component {
	fetchData = ( input, callback ) => {
		const {
			args: inputArgs,
		} = this.props.restInfo;

		const finalArgs = {
			...inputArgs,
			search: sprintf( inputArgs.search, this.sanitizeInput( input ) ),
		};

		const path = this.props.restInfo.base + stringify( finalArgs, true );

		// @deprecated use wp.apiFetch instead
		this.suggestionsRequest = wp.apiFetch( { path } );

		return this.suggestionsRequest
			.then(
				( items ) => {
					const finalItems = items.reduce(
						( result, item ) => {
							if (
								! item.id &&
								! ( item.name || item.title ) // "name" is for the Toolset Rest API endpoint while "title" is for the WP REST API endpoint.
							) {
								return result;
							}

							result.push(
								{
									value: item.id,
									label: item.name || item.title,
								}
							);

							return result;
						},
						[]
					);

					return finalItems;
				},
				( xhr ) => {
					if ( xhr.statusText === 'abort' ) {
						callback( [] );
					}
				}
			)
			.then( ( items ) => {
				callback( items );
			} );
	};

	debouncedFetchData = debounce( this.fetchData, 500 );

	getRestData = ( input, callback ) => {
		if ( input && input.length >= 2 ) {
			this.debouncedFetchData( input, callback );
		}
	};

	sanitizeInput = ( input ) => {
		let output = input
			.replace( /[^\w\s\d]/gi, '' )
			.replace( /[`~!@#$%^&*()_|+\-=?;:'",.<>\{\}\[\]\\\/]/gi, '' )
			.split( ' ' )
			.join( '-' );
		if ( '' === output ) {
			output = 'null';
		}
		return output;
	};

	render() {
		const {
			options,
			value,
			placeholder,
			styles,
			isClearable,
			id,
			className,
			onChange,
		} = this.props;

		const ReactSelectComponent = this.props.restInfo ? AsyncSelect : Select;
		const props = {
			value,
			placeholder,
			styles,
			id,
			className,
			isClearable,
			onChange,
		};

		if ( this.props.restInfo ) {
			if ( this.props.defaultOptions ) {
				props.defaultOptions = this.props.defaultOptions;
			}
			props.loadOptions = this.getRestData;
		} else {
			props.options = options;
		}

		return <ReactSelectComponent { ...props } />;
	}
}

ReactSelect.defaultProps = {
	options: [],
	defaultOptions: [],
	value: {},
	placeholder: __( 'Select...', 'wpv-views' ),
	styles: {},
	isClearable: false,
	onChange: () => {
		return null;
	},
	restInfo: null,
};

ReactSelect.propTypes = {
	options: PropTypes.array,
	defaultOptions: PropTypes.array,
	value: PropTypes.object,
	placeholder: PropTypes.string,
	styles: PropTypes.object,
	isClearable: PropTypes.bool,
	id: PropTypes.string,
	className: PropTypes.string,
	onChange: PropTypes.func.isRequired,
};

export {
	ReactSelect as ReactSelect,
	AsyncSelect,
};
