<?php
namespace OTGS\Toolset\Views\Controller\Compatibility;

use \OTGS\Toolset\Views\Controller\Compatibility as Compatibility;
use \OTGS\Toolset\Views\Services\WPMLService;

/**
 * Class BlockEditorWPA
 *
 * This class enables editing WordPress Archives using the View block from within the new editor. As the View block needs
 * to be hosted inside a post, a new helper post type ("WPA Helper") is needed for this.
 * Along with the creation of the normal WPA post, the relevant WPA Helper post is created as well. Those two are linked
 * with a parent-child relationship.
 * As soon as both posts have been created, instead of been transferred to the old editor, the user is transferred to the
 * post edit screen of the WPA Helper post which by default contains an empty WPA block, already linked to the WPA created
 * moments before. This block is then enhanced with all the relevant default properties that a View block has, adjusted for
 * use in WordPress Archives.
 * While the WPA is formed in the block, the block also provides a preview of the output. The preview is constantly been
 * hijacked to always show the proper loop in the preview, according a preview loop selector on the Settings Sidebar.
 * The frontend rendering process is not affected at all. Since the WPA post has all the proper attributes and properties
 * coming from the backend, it keeps working as it previously did.
 *
 * @package OTGS\Toolset\Views\Controller\Compatibility
 *
 * @since 3.0
 */
class BlockEditorWPA extends Compatibility\Base {
	/** @var Toolset_Constants */
	private $constants;

	/** @var Toolset_Renderer */
	private $toolset_renderer;

	/** @var Toolset_Output_Template_Repository */
	private $template_repository;

	/** @var \WPV_Settings */
	private $wpv_settings;

	/** @var \WPV_WordPress_Archive_Frontend */
	private $wpv_view_archive_loop;

	/** @var \WPV_Editor_Loop_Selection */
	private $wpv_editor_loop_selection;

	/** @var array */
	private $wpv_wordpress_archive_get_instance;

	/** @var array */
	private $view_base_get_instance;

	const WPA_HELPER_POST_TYPE = 'wpa-helper';
	const WPA_EDITOR_META_KEY = '_wpa_editor';

	public function __construct(
		\Toolset_Constants $constants,
		\Toolset_Renderer $toolset_renderer,
		\WPV_Output_Template_Repository $template_repository,
		\WPV_WordPress_Archive_Frontend $wpv_view_archive_loop,
		\WPV_Settings $wpv_settings,
		\WPV_Editor_Loop_Selection $wpv_editor_loop_selection,
		array $wpv_wordpress_archive_get_instance,
		array $view_base_get_instance
	) {
		$this->constants = $constants;
		$this->toolset_renderer = $toolset_renderer;
		$this->template_repository = $template_repository;
		$this->wpv_view_archive_loop = $wpv_view_archive_loop;
		$this->wpv_settings = $wpv_settings;
		$this->wpv_wordpress_archive_get_instance = $wpv_wordpress_archive_get_instance;
		$this->view_base_get_instance = $view_base_get_instance;
		$this->wpv_editor_loop_selection = $wpv_editor_loop_selection;
	}

	public function initialize() {
		// The WPA helper post type needs to be registered no matter what as otherwise PHP Notices occur when the DS cache,
		// tries to fetch the cache for such posts using REST request.
		add_filter( 'init', array( $this, 'register_wpa_helper_post_type' ) );

		// The WPA helper post type needs to be excluded from the Toolset own post types no matter what as otherwise they
		// will show up in several places for example the Custom Field Group post type assignment panel.
		add_filter( 'toolset_filter_exclude_own_post_types', array( $this, 'add_wpa_helper_post_type_in_toolset_exclude_list' ) );

		add_action( 'admin_init', array( $this, 'redirect_wpa_helper_listing_page' ) );

		add_filter( 'register_post_type_args', array( $this, 'make_wpa_block_editor_editable' ), 10, 2 );

		add_filter( 'wpv_filter_wpv_disable_post_content_template_metabox', array( $this, 'remove_the_content_template_metabox' ), 10, 2 );

		add_filter( 'wpv_filter_wpa_edit_link', array( $this, 'convert_wpa_edit_link' ), 10, 2 );

		add_action( 'wpv_action_before_render_view_editor_shortcode', array( $this, 'maybe_set_wpa_query' ) );

		add_action( 'wpv_action_after_render_view_editor_shortcode', array( $this, 'maybe_revert_in_loop' ) );

		add_filter( 'wpv_filter_view_editor_first_item_id', array( $this, 'maybe_set_wpa_first_item_id' ), 10, 2 );

		add_action( 'before_delete_post', array( $this, 'before_delete_wpa_preview_post' ), 20 );

		add_action( 'before_delete_post', array( $this, 'scan_and_delete_orphaned_preview_posts' ), 20 );

		add_filter( 'wpv_filter_localize_view_editor_assets', array( $this, 'add_maybe_is_wpa_helper_info' ) );

		add_filter( 'wpv_filter_localize_view_block_strings', array( $this, 'maybe_add_loop_selection_options' ) );

		add_filter( 'wpv_filter_localize_view_block_strings', array( $this, 'maybe_add_loop_links' ) );

		add_filter( 'wpv_filter_localize_view_block_strings', array( $this, 'add_switch_to_classic_url' ) );

		add_filter( 'wpv_filter_localize_view_block_strings', array( $this, 'add_theme_settings' ) );

		add_action( 'wpv_action_after_save_views_from_previews', array( $this, 'maybe_update_loop_selection' ), 10, 2 );

		add_filter( 'wpv_filter_view_block_data_from_db', array( $this, 'adjust_view_block_data_from_db' ), 10, 2 );

		add_action( 'admin_init', array( $this, 'switch_editor' ) );

		add_filter( 'wpv_filter_converted_view_settings_for_backend', array( $this, 'convert_view_settings_for_wpa' ), 10, 2 );

		add_action( 'admin_head', array( $this, 'remove_all_metaboxes' ), PHP_INT_MAX );

		add_filter( 'toolset_theme_settings_force_backend_editor', array( $this, 'enable_theme_settings_integration' ) );

		add_filter( 'wpv_filter_override_view_layout_settings', array( $this, 'adapt_settings_for_translation' ), 10, 2 );

		// Compatibility: Adjust the frontend post type link for WordPress Archive helper posts. It changes the link to the one
		// of the first assigned loop.
		add_filter( 'post_type_link', array( $this, 'adjust_view_post_type_link' ), 10, 2 );

		// Compatibility: Remove the WPA Helper link if "adjust_view_post_type_link" produced an empty or invalid link.
		add_filter( 'wpml_document_view_item_link', array( $this, 'maybe_delete_wpa_helper_view_link' ), 10, 5 );

		if ( \Toolset_User_Editors_Editor_Gutenberg::GUTENBERG_SCREEN_ID !== $this->wpv_settings->get_raw_value( 'default_wpa_editor', false ) ) {
			return;
		}

		// Tasks only available when default editor is Gutenberg
		add_filter( 'wpv_filter_wpa_created', array( $this, 'create_wpa_helper_post' ), 10, 2 );
	}

	public function make_wpa_block_editor_editable( $args, $name ) {
		if ( 'view' === $name ) {
			$args['show_ui'] = true;
			$args['show_in_rest'] = true;
		}
		return $args;
	}

	private function is_wpa( $view_id ) {
		if (
			$view_id <= 0 ||
			! \WPV_View_Base::is_archive_view( $view_id )
		) {
			return false;
		}

		return true;
	}

	private function is_wpa_helper( $post ) {
		return self::WPA_HELPER_POST_TYPE === $post->post_type;
	}

	private function get_wpa_helper_post_content() {
		$renderer = $this->toolset_renderer;
		$wpa_content = $renderer->render(
			$this->template_repository->get( $this->constants->constant( 'WPV_Output_Template_Repository::WPA_EDITOR_BLOCK_DEFAULT_MARKUP' ) ),
			array(),
			false
		);

		return $wpa_content;
	}

	/**
	 * Provides the WPA Helper post ID for the given WordPress Archive post ID.
	 *
	 * @param $wpa_id
	 *
	 * @return int
	 */
	private function maybe_get_helper_post_id( $wpa_id ) {
		if ( ! $wpa_id ) {
			return 0;
		}

		$args = array(
			'post_parent' => $wpa_id,
			'post_type' => self::WPA_HELPER_POST_TYPE,
		);

		$children = get_children( $args );

		if ( empty( $children ) ) {
			return (int) $wpa_id;
		}

		return (int) array_keys( $children )[0];
	}

	/**
	 * Converts the WordPress Archive edit links in the WordPress Archive listing page.
	 *
	 * @param $wpa_edit_link
	 * @param null $wpa_id
	 *
	 * @return string|void|null
	 */
	public function convert_wpa_edit_link( $wpa_edit_link, $wpa_id = null ) {
		// Legacy name variable : WordPress.NamingConventions.ValidVariableName.VariableNotSnakeCase .
		global $WPV_settings; // phpcs:ignore

		$edit_link = $wpa_edit_link;
		if (
			( ! $wpa_id && \Toolset_User_Editors_Editor_Gutenberg::GUTENBERG_SCREEN_ID === $WPV_settings->default_wpa_editor ) || // phpcs:ignore
			$this->is_gutenberg_editor( $wpa_id )
		) {
			$wpa_helper_id = $this->maybe_get_helper_post_id( $wpa_id );
			// Pass a context to get_edit_post_link different than 'display',
			// needed for the frontend admin bar WPA creation.
			$edit_link = $wpa_helper_id > 0 ? get_edit_post_link( $wpa_helper_id, 'link' ) : admin_url( 'post.php?action=edit&post=' );
		}

		return $edit_link;
	}

	/**
	 * Creates the WPA Helper post for a given WordPress Archive post.
	 *
	 * @param $wpa_id
	 * @param $wpa
	 *
	 * @return mixed
	 */
	public function create_wpa_helper_post( $wpa_id, $wpa ) {
		$helper_post = array(
			'post_title' => $wpa->title,
			'post_name' => $wpa->slug . '-' . self::WPA_HELPER_POST_TYPE,
			'post_content' => $this->get_wpa_helper_post_content(),
			'post_status' => 'publish',
			'post_type' => self::WPA_HELPER_POST_TYPE,
			'post_parent' => $wpa_id,
		);

		$wpa_helper_post_id = wp_insert_post( $helper_post );
		update_post_meta( $wpa_id, self::WPA_EDITOR_META_KEY, \Toolset_User_Editors_Editor_Gutenberg::GUTENBERG_SCREEN_ID );

		return $wpa_helper_post_id ?: $wpa_id;
	}

	/**
	 * Registers the WPA Helper post type.
	 */
	public function register_wpa_helper_post_type() {
		$labels = array(
			'name' => _x( 'WordPress Archives Blocks', 'Post Type General Name', 'wpv-views' ),
			'singular_name' => _x( 'WordPress Archives Block', 'Post Type Singular Name', 'wpv-views' ),
			'menu_name' => _x( 'WordPress Archives Blocks', 'Admin Menu text', 'wpv-views' ),
			'name_admin_bar' => _x( 'WordPress Archives Blocks', 'Add New on Toolbar', 'wpv-views' ),
			'archives' => __( 'WordPress Archives Blocks Archives', 'wpv-views' ),
			'attributes' => __( 'WordPress Archives Blocks Attributes', 'wpv-views' ),
			'parent_item_colon' => __( 'Parent WordPress Archives Block:', 'wpv-views' ),
			'all_items' => __( 'All WordPress Archives Blocks', 'wpv-views' ),
			'add_new_item' => __( 'Add New WordPress Archives Block', 'wpv-views' ),
			'add_new' => __( 'Add New', 'wpv-views' ),
			'new_item' => __( 'New WordPress Archives Block', 'wpv-views' ),
			'edit_item' => __( 'Edit WordPress Archives Block', 'wpv-views' ),
			'update_item' => __( 'Update WordPress Archives Block', 'wpv-views' ),
			'view_item' => __( 'View WordPress Archives Block', 'wpv-views' ),
			'view_items' => __( 'View WordPress Archives Blocks', 'wpv-views' ),
			'search_items' => __( 'Search WordPress Archives Blocks', 'wpv-views' ),
			'not_found' => __( 'Not found', 'wpv-views' ),
			'not_found_in_trash' => __( 'Not found in Trash', 'wpv-views' ),
			'featured_image' => __( 'Featured Image', 'wpv-views' ),
			'set_featured_image' => __( 'Set featured image', 'wpv-views' ),
			'remove_featured_image' => __( 'Remove featured image', 'wpv-views' ),
			'use_featured_image' => __( 'Use as featured image', 'wpv-views' ),
			'insert_into_item' => __( 'Insert into WordPress Archives Block', 'wpv-views' ),
			'uploaded_to_this_item' => __( 'Uploaded to this WordPress Archives Block', 'wpv-views' ),
			'items_list' => __( 'WordPress Archives Blocks list', 'wpv-views' ),
			'items_list_navigation' => __( 'WordPress Archives Blocks list navigation', 'wpv-views' ),
			'filter_items_list' => __( 'Filter WordPress Archives Blocks list', 'wpv-views' ),
		);
		$args = array(
			'label' => __( 'WordPress Archives Blocks', 'wpv-views' ),
			'description' => __( '', 'wpv-views' ),
			'labels' => $labels,
			'menu_icon' => '',
			'supports' => array( 'title', 'editor', 'author' ),
			'taxonomies' => array(),
			'public' => false,
			'show_ui' => true,
			'show_in_menu' => false,
			'menu_position' => 5,
			'show_in_admin_bar' => false,
			'show_in_nav_menus' => false,
			'can_export' => false,
			'has_archive' => false,
			'hierarchical' => true,
			'exclude_from_search' => true,
			'show_in_rest' => true,
			'publicly_queryable' => false,
			'capability_type' => 'post',
		);
		register_post_type( self::WPA_HELPER_POST_TYPE, $args );
	}

	/**
	 * Adds the WPA helper post to the Toolset Exclude list of post types.
	 *
	 * @param $exclude_list
	 *
	 * @return array
	 */
	public function add_wpa_helper_post_type_in_toolset_exclude_list( $exclude_list ) {
		$exclude_list[] = self::WPA_HELPER_POST_TYPE;

		return $exclude_list;
	}

	/**
	 * Converts the View settings array into WordPress Archive settings.
	 *
	 * @param $view_settings
	 * @param $view_data
	 *
	 * @return mixed
	 */
	public function convert_view_settings_for_wpa( $view_settings, $view_data ) {
		$view_id = (int) toolset_getnest( $view_data, array( 'general', 'id' ), 0 );
		if (
			toolset_getarr( $view_data, 'is_wpa', false ) ||
			$this->is_wpa( $view_id )
		) {
			$view_settings['view-query-mode'] = 'archive';
			$wpa = call_user_func( $this->wpv_wordpress_archive_get_instance, $view_id );
			$view_settings[ \WPV_View_Embedded::VIEW_SETTINGS_PURPOSE ] = $wpa->view_settings[ \WPV_View_Embedded::VIEW_SETTINGS_PURPOSE ];
		}

		return $view_settings;
	}

	/**
	 * Removes the Content Template metabox from the WPA Helper edit page.
	 *
	 * @param $status
	 * @param $post
	 *
	 * @return bool
	 */
	public function remove_the_content_template_metabox( $status, $post ) {
		if ( self::WPA_HELPER_POST_TYPE === $post->post_type ) {
			return true;
		}

		return $status;
	}

	/**
	 * Sets the needed parameters for the WordPress Archive to be rendered properly on the editor preview.
	 *
	 * @param $wpa_id
	 */
	public function maybe_set_wpa_query( $wpa_id ) {
		if ( ! $this->is_wpa( $wpa_id ) ) {
			return;
		}

		$wpa = call_user_func( $this->wpv_wordpress_archive_get_instance, $wpa_id );
		$wpa_data = $wpa->get_postmeta( '_wpv_view_data' );

		$this->wpv_view_archive_loop->in_the_loop = true;
		$args = array(
			'posts_per_page' => toolset_getnest( $wpa_data, array( 'pagination', 'page_size' ) ),
		);

		$selected_loop = toolset_getnest( $wpa_data, array( 'content_selection', 'wpaLoopsPreview' ) );
		$assigned_loops = toolset_getnest( $wpa_data, array( 'content_selection', 'wpaLoops' ) );

		// For the cases where the preview loop hasn't been set or the set loop is no longer in the list of assigned loops...
		if (
			! $selected_loop ||
			! in_array( $selected_loop, $assigned_loops, true )
		) {
			// ...select the first loop in the assigned list or null.
			$selected_loop = isset( $assigned_loops[0] ) ? $assigned_loops[0] : null;
		}

		// The selected loop is a post type, so the query will fetch only posts of this post type.
		if ( get_post_type_object( $selected_loop ) ) {
			$args['post_type'] = $selected_loop;
		}

		// The selected loop is a taxonomy, so the query will fetch only posts related to this taxonomy.
		if ( get_taxonomy( $selected_loop ) ) {
			// First the ID of a term with posts is retrieved.
			$terms_with_posts = get_terms(
				$selected_loop,
				array(
					'hide_empty' => 1,
					'number' => 1,
				)
			);

			// If the retrieved term has posts...
			if (
				false === $terms_with_posts instanceof WP_Error &&
				is_array( $terms_with_posts ) &&
				! empty( $terms_with_posts )
			) {
				// ...the retrieved term slug is fed to the query in order to get posts that has that term assigned to them.
				$terms_with_posts = array_values( $terms_with_posts );
				$term = array_shift( $terms_with_posts );
				$args['tax_query'] = array(
					array(
						'taxonomy' => $selected_loop,
						'field' => 'slug',
						'terms' => $term,
					),
				);
			}
		}

		if ( 'author-page' === $selected_loop ) {
			// todo: Make the author for the author archive page preview dynamic.
			// Currently, the currently logged-in user ID is used to build the preview for the author page, but a select
			// control offering the site's users needs to be placed in the inspector of the block.
			$args['author'] = get_current_user_id();
		}

		$this->wpv_view_archive_loop->query = new \WP_Query( $args );
	}

	/**
	 * Reverts "in_the_loop" property for the WordPress Archive loop.
	 *
	 * This is needed because on the editor when the preview of a WordPress Archive query is built to be previewed, the
	 * "in_the_loop" property is manually set to "true".
	 *
	 * @param $wpa_id
	 */
	public function maybe_revert_in_loop( $wpa_id  ) {
		if ( ! $this->is_wpa( $wpa_id ) ) {
			return;
		}

		$this->wpv_view_archive_loop->in_the_loop = false;
	}

	/**
	 * Sets the first item ID for the WordPress Archive block in order to render the WordPress Archive in proper order.
	 *
	 * @param $first_item_id
	 * @param $wpa_id
	 *
	 * @return mixed
	 */
	public function maybe_set_wpa_first_item_id( $first_item_id, $wpa_id ) {
		if ( ! $this->is_wpa( $wpa_id ) ) {
			return $first_item_id;
		}

		$wpa_posts = $this->wpv_view_archive_loop->query->posts;
		if ( count( $wpa_posts ) ) {
			$first_item_id = $wpa_posts[0]->ID;
		}

		return $first_item_id;
	}

	/**
	 * Handles the deletion of the WPA Helper post whenever a WordPress Archive post is been deleted.
	 *
	 * @param $wpa_id
	 */
	public function before_delete_wpa_preview_post( $wpa_id ) {
		if ( ! $this->is_wpa( $wpa_id ) || ! $this->is_gutenberg_editor( $wpa_id ) ) {
			return;
		}

		$args = array(
			'post_parent' => $wpa_id,
			'post_type' => BlockEditorWPA::WPA_HELPER_POST_TYPE,
		);

		$wpa_helper = get_posts( $args );

		if (
			is_array( $wpa_helper ) &&
			count( $wpa_helper ) > 0
		) {
			// Delete all the Children of the Parent Page
			foreach ( $wpa_helper as $wpa_helper_post ) {
				wp_delete_post( $wpa_helper_post->ID, true );
			}
		}
	}

	/**
	 * Scans for orphaned preview post for the given WPA and deletes them if any.
	 *
	 * The WordPress Archive block links to the relevant preview post upon WPA Helper post save. If this doesn't happen
	 * and the page is refreshed before that, the created preview post remains orphan and when the page loads a new one
	 * is created.
	 *
	 * @param int $wpa_id ID of the WPA whose preview posts should be scanned.
	 */
	public function scan_and_delete_orphaned_preview_posts( $wpa_id ) {
		if ( ! $this->is_wpa( $wpa_id ) ) {
			return;
		}

		$wpa = call_user_func( $this->wpv_wordpress_archive_get_instance, $wpa_id );

		if (
			'trash' == $wpa->post_status
			&& '__trashed' === substr( $wpa->slug, -9 )
		) {
			$wpa_slug = substr( $wpa->slug, 0, -9 );
		} else {
			$wpa_slug = $wpa->slug;
		}

		$args = array(
			'post_type' => \WPV_View_Base::POST_TYPE,
			'post_status' => 'draft',
			'meta_query' => array(
				array(
					'key' => \OTGS\Toolset\Views\Controllers\V1\Wpa::WPA_PREVIEW_OF_META_KEY,
					'value' => $wpa_slug,
					'compare' => '=',
				),
			),
		);

		$other_preview_posts = new \WP_Query( $args );
		foreach ( $other_preview_posts->get_posts() as $other_preview_post ) {
			wp_delete_post( $other_preview_post->ID );
		}
	}

	/**
	 * Specifies in the View/WordPress Archive block localization data whether the current block is a View or a WordPress
	 * Archive block.
	 *
	 * @param  array $localization_data
	 *
	 * @return array
	 */
	public function add_maybe_is_wpa_helper_info( $localization_data ) {
		global $post;

		$localization_data['isWpaHelper'] = $post && $this->is_wpa_helper( $post );

		return $localization_data;
	}

	/**
	 * Pre-populates the available Loops and adds them to the localization data of the View/WordPress Archive block.
	 *
	 * @param  array $localization_data
	 *
	 * @return array
	 */
	public function maybe_add_loop_selection_options( $localization_data ) {
		$localization_data[ 'loopSelectionOptions' ] = $this->get_wpa_loops();
		return $localization_data;
	}

	/*
	 * Populates the available Loops.
	 */
	private function get_wpa_loops() {
		$wp_loops = \WPV_Editor_Loop_Selection::get_wp_loops();
		$pt_loops = \WPV_Editor_Loop_Selection::get_pt_loops();
		$taxonomy_loops = array_map(
			function( $taxonomy ) {
				return (array) $taxonomy;
			},
			\WPV_Editor_Loop_Selection::get_taxonomy_loops()
		);


		$result = array(
			'wordpressLoops' => array(),
			'postTypeLoops' => array(),
			'taxonomyLoops' => array(),
		);

		foreach ( $wp_loops as $key => $wp_loop ) {
			$result['wordpressLoops'][] = array(
				'value' => $key,
				'label' => $wp_loop['display_name'],
				'name' => 'wpv-view-loop-' . $key,
				'url' => $wp_loop[ 'archive_url' ],
			);
		}

		foreach ( $pt_loops as $pt_loops ) {
			$result['postTypeLoops'][] = array(
				'value' => $pt_loops['name'],
				'label' => $pt_loops['display_name'],
				'name' => 'wpv-view-loop-' . $pt_loops['loop'],
				'url' => $pt_loops[ 'archive_url' ],
			);
		}

		foreach ( $taxonomy_loops as $taxonomy_loop ) {
			// Get ID of a term that has some posts, if such term exists.
			$terms_with_posts = get_terms(
				$taxonomy_loop['name'],
				array(
					'hide_empty' => 1,
					'number' => 1,
					)
			);

			if (
				$terms_with_posts instanceof WP_Error ||
				! is_array( $terms_with_posts ) ||
				empty( $terms_with_posts )
			) {
				$tax_link = null;
			} else {
				$terms_with_posts = array_values( $terms_with_posts );
				$term = array_shift( $terms_with_posts );
				$tax_link = get_term_link( $term, $taxonomy_loop['name'] );
			}

			$result['taxonomyLoops'][] = array(
				'value' => $taxonomy_loop['name'],
				'label' => $taxonomy_loop['label'],
				'singular' => $taxonomy_loop['labels']->singular_name,
				'name' => 'wpv-view-taxonomy-loop-' . $taxonomy_loop['name'],
				'url' => $tax_link,
			);
		}

		return $result;
	}

	/**
	 * Flattens the loops array.
	 *
	 * @return array
	 */
	private function get_flattened_wpa_loops() {
		$loops = $this->get_wpa_loops();
		return array_merge(
			$loops['wordpressLoops'],
			$loops['postTypeLoops'],
			$loops['taxonomyLoops']
		);
	}

	/**
	 * Handles the saving of the Loop selection whenever the WordPress Archive block is saved.
	 * @param $wpa_id
	 * @param $wpa_data
	 */
	public function maybe_update_loop_selection( $wpa_id, $wpa_data ) {
		if (
			( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) ||
			! $this->is_wpa( $wpa_id ) ||
			! $this->is_gutenberg_editor( $wpa_id ) ||
			'publish' !== get_post_status( $wpa_id ) || // The loops won't be updated for the WordPress Archive preview post which is in "draft" status.
			toolset_getnest( $wpa_data, array( 'general', 'id' ), false ) !== $wpa_id
		) {
			return;
		}

		$loops = $this->get_flattened_wpa_loops();

		$assigned_loops = toolset_getnest( $wpa_data, array( 'content_selection', 'wpaLoops' ) );

		$assigned_loops_for_saving = array_reduce(
			$assigned_loops,
			function( $result, $assigned_loop ) use ( $loops ) {
				$maybe_loop_index = array_search( $assigned_loop, array_column( $loops, 'value' ), true );
				if ( false !== $maybe_loop_index ) {
					$result[ $loops[ $maybe_loop_index ]['name'] ] = 'on';
				}
				return $result;
			},
			array()
		);

		$this->wpv_view_archive_loop->update_view_archive_settings( $wpa_id, $assigned_loops_for_saving );
	}

	public function adjust_view_block_data_from_db( $view_data, $wpa_id ) {
		$wpa = call_user_func( $this->view_base_get_instance, $wpa_id );
		// we need to set the view ID in all cases, because it could be changed on import/export and recreated from slug
		$view_data['general']['id'] = $wpa_id;
		if ( $wpa->is_a_wordpress_archive() ) {
			$assigned_loops = array_map(
				function( $item ) {
					return isset( $item['post_type_name'] ) ? $item['post_type_name'] : $item['slug'];
				},
				$wpa->get_assigned_loops()
			);

			$view_data['general']['name'] = $wpa->title;
			$view_data['general']['slug'] = $wpa->slug;
			$view_data['general']['isWpa'] = true;
			$view_data['pagination']['enable_pagination'] = true;
			$view_data['content_selection']['wpaLoops'] = $assigned_loops;
			$default_preview_loop = isset( $assigned_loops[0] ) ? $assigned_loops[0] : null;
			$view_data['content_selection']['wpaLoopsPreview'] = toolset_getnest( $view_data, array( 'content_selection', 'wpaLoopsPreview' ), $default_preview_loop );
			$view_data['loop']['wizard_done'] = true;
			$view_data['loop']['wizard_step'] = 3;
		}

		return $view_data;
	}

	public function maybe_add_loop_links( $localization_data ) {
		$loops = $this->get_flattened_wpa_loops();
		$links = array();

		foreach ( $loops as $loop ) {
			$links[ $loop['value'] ] = $loop['url'];
		}

		$localization_data['archiveLinks'] = $links;

		return $localization_data;
	}

	/**
	 * It is triggered when a post is trashed. Checks if the trashed post is a WPA Helper post and if so, it reverts back
	 * to its proper state ("published") and handles the trashing of the WPA post it's linked to.
	 * The trashing, apart from the status change for the WPA post, also clears the assignment of it to any post loops.
	 */
	private function maybe_wpa_helper_post_trashed() {
		$is_trashed = sanitize_text_field( toolset_getget( 'trashed', false ) );

		if ( ! $is_trashed ) {
			return;
		}

		$post_ids = explode( ',', sanitize_text_field( toolset_getget( 'ids', array() ) ) );
		if ( count( $post_ids ) > 1 ) {
			return;
		}

		$post_id = $post_ids[0];

		if ( self::WPA_HELPER_POST_TYPE === get_post_type( $post_id ) ) {
			// Restore the WPA helper post to its proper state (always "published").
			wp_update_post(
				array(
					'ID' => $post_id,
					'post_status' => 'publish',
				)
			);

			// Trash the actual WPA post.
			$wpa_helper_post = get_post( $post_id );
			wp_update_post(
				array(
					'ID' => $wpa_helper_post->post_parent,
					'post_status' => 'trash',
				)
			);

			// Remove any assignments to loops for the WPA post.
			$settings_array = $this->wpv_settings->get();
			if ( ! empty( $settings_array ) ) {
				foreach ( $settings_array as $option_name => $option_value ) {
					if (
						0 === strpos( $option_name, 'view_' ) &&
						(int) $option_value === (int) $wpa_helper_post->post_parent
					) {
						$this->wpv_settings[ $option_name ] = 0;
					}
				}
				$this->wpv_settings->save();
			}
		}
	}

	/**
	 * Returns if the WPA has been created using Gutenberg editor
	 *
	 * @param int $wpa_id WPA ID.
	 * @return boolean
	 */
	private function is_gutenberg_editor( $wpa_id ) {
		$wpa_editor = get_post_meta( $wpa_id, self::WPA_EDITOR_META_KEY, true );
		return \Toolset_User_Editors_Editor_Gutenberg::GUTENBERG_SCREEN_ID === $wpa_editor;
	}

	/**
	 * Handles WPA switch editors
	 */
	public function switch_editor() {
		$change_editor = toolset_getget( 'wpa_change_editor' );
		if ( \Toolset_User_Editors_Editor_Gutenberg::GUTENBERG_SCREEN_ID === $change_editor ) {
			$this->switch_to_classic_editor();
		} elseif ( 'basic' === $change_editor ) {
			$this->switch_to_block_editor();
		}

	}

	/**
	 * Switches from GB WPA to a classic one
	 */
	private function switch_to_classic_editor() {
		$wpa_id = toolset_getget( 'view_id' );
		if ( $this->is_gutenberg_editor( $wpa_id ) ) {
			// Handles wpa-helper.
			$this->before_delete_wpa_preview_post( $wpa_id );
			delete_post_meta( $wpa_id, self::WPA_EDITOR_META_KEY );

			// Handles wpa settings.
			$meta = get_post_meta( $wpa_id, '_wpv_layout_settings', true );
			$empty_loop_output = \WPV_View_Base::generate_loop_output();
			$meta['layout_meta_html'] = $empty_loop_output['loop_output_settings']['layout_meta_html'];
			update_post_meta( $wpa_id, '_wpv_layout_settings', $meta );

			$snapshot = get_post_meta( $wpa_id, '_wpa_snapshot', true );
			delete_post_meta( $wpa_id, '_wpv_view_data' );
			delete_post_meta( $wpa_id, '_wpa_snapshot' );

			if ( $snapshot ) {
				$meta['real_fields'] = $snapshot;
				update_post_meta( $wpa_id, '_wpv_layout_settings', $meta );
			}

			$settings = get_post_meta( $wpa_id, '_wpv_settings', true );
			$settings['view-query-mode'] = 'archive';
			update_post_meta( $wpa_id, '_wpv_settings', $settings );

			// Handles wpa content.
			$wpa = get_post( $wpa_id );
			$wpa->post_content = "[wpv-layout-meta-html]\n[wpv-filter-meta-html]";
			wp_update_post( $wpa );
		}
	}

	/**
	 * Redirects the user if he tries to access the WPA Helper post listing page.
	 */
	public function redirect_wpa_helper_listing_page() {
		global $pagenow;
		/* Check current admin page. */
		if (
			'edit.php' === $pagenow &&
			self::WPA_HELPER_POST_TYPE === sanitize_text_field( toolset_getget( 'post_type', '' ) )
		) {
			$this->maybe_wpa_helper_post_trashed();

			$redirect_url = add_query_arg(
				array(
					'page' => 'view-archives',
				),
				admin_url( 'admin.php' )
			);

			wp_safe_redirect( $redirect_url );
			exit;
		}
	}

	/**
	 * Switches from Classic editor to GB
	 */
	private function switch_to_block_editor() {
		$wpa_id = toolset_getget( 'view_id' );
		if ( ! $this->is_gutenberg_editor( $wpa_id ) ) {
			update_post_meta( $wpa_id, self::WPA_EDITOR_META_KEY, \Toolset_User_Editors_Editor_Gutenberg::GUTENBERG_SCREEN_ID );
			$wpa = get_post( $wpa_id );
			// Normalize data.
			$wpa->title = $wpa->post_title;
			$wpa->slug  = $wpa->post_name;
			$wp_helper_id = $this->create_wpa_helper_post( $wpa_id, $wpa );
			$meta = get_post_meta( $wpa_id, '_wpv_layout_settings', true );
			if ( isset( $meta['real_fields'] ) ) {
				update_post_meta( $wpa_id, '_wpa_snapshot', $meta['real_fields'] );
			}

			$settings = get_post_meta( $wpa_id, '_wpv_settings', true );
			$settings['view-query-mode'] = 'archive';
			update_post_meta( $wpa_id, '_wpv_settings', $settings );

			$edit_link = get_edit_post_link( $wp_helper_id, 'none' );
			wp_safe_redirect( $edit_link );
			exit;
		}
	}

	/**
	 * Adds the URL for switching the WordPress Archive editor for the current WordPress Archive back to the legacy one.
	 *
	 * @param  array $localization_data
	 *
	 * @return array
	 */
	public function add_switch_to_classic_url( $localization_data ) {
		global $post;

		if (
			! $post ||
			! isset( $post->post_parent )
		) {
			return $localization_data;
		}

		$localization_data['switchToClassicUrl'] = admin_url( 'admin.php?page=view-archives-editor&view_id=' . $post->post_parent ) . '&wpa_change_editor=gutenberg';
		$localization_data['includeSwitchToClassicUrl'] = ( 'mixed' === wpv_get_views_editing_experience() ) ? '1' : '0';
		return $localization_data;
	}

	/**
	 * Remove all the meta boxes because first no meta boxes are needed when editing a WPA but also interferes with the proper
	 * execution of the WPA Helper trash post workflow.
	 *
	 * The WPA helper post is marked as trashed, then the actual WPA post is marked as trashed and the helper post is reverted
	 * to public, but the when GB saves the meta box data, the helper post is turned again back to trashed, because GB!
	 */
	public function remove_all_metaboxes() {
		global $wp_meta_boxes;

		// If WPML is active then let translation metabox.
		if ( function_exists( 'icl_object_id' ) && isset( $wp_meta_boxes[ self::WPA_HELPER_POST_TYPE ]['side']['high']['icl_div'] ) ) {
			$wp_meta_boxes[ self::WPA_HELPER_POST_TYPE ] = [ // phpcs:ignore
				'side' => [
					'high' => [ 'icl_div' => $wp_meta_boxes[ self::WPA_HELPER_POST_TYPE ]['side']['high']['icl_div'] ],
				],
			];
		} else {
			// Removing all the meta boxes for the WordPress Archive helper post type.
			unset( $wp_meta_boxes[ self::WPA_HELPER_POST_TYPE ] );
		}
	}

	/**
	 * Modifies WPA settings if WPML is installed
	 *
	 * @param array $settings View settings.
	 * @param int   $wpa_id WPA ID.
	 * @return array
	 */
	public function adapt_settings_for_translation( $settings, $wpa_id ) {
		// If WPML is active.
		$wpml_active_and_configured = apply_filters( 'wpml_setting', false, 'setup_complete' );
		if ( ! is_admin() && $wpml_active_and_configured && $this->is_wpa( $wpa_id ) && isset( $settings['layout_meta_html'] ) ) {
			$helper_id = $this->maybe_get_helper_post_id( $wpa_id );
			$translated_helper_id = apply_filters( 'wpml_object_id', $helper_id, self::WPA_HELPER_POST_TYPE, true );
			if ( $helper_id !== $translated_helper_id ) {
				$translated_helper = get_post( $translated_helper_id );
				$settings = WPMLService::updateSettingsFromHtml( $translated_helper->post_content, $settings );
			}
		}
		return $settings;
	}

	/**
	 * Fakes the enabling of the theme settings integration for the case where a WPA helper post is edited, in order to
	 * offer the theme settings on the WPA block.
	 *
	 * @param  bool $condition
	 *
	 * @return bool
	 */
	public function enable_theme_settings_integration( $condition ) {
		if ( ! is_admin() ) {
			return $condition;
		}

		$action = sanitize_text_field( toolset_getget( 'action', false ) );

		if ( ! $action || 'edit' !== $action ) {
			return $condition;
		}

		$post_id = sanitize_text_field( toolset_getget( 'post', false ) );

		if ( ! $post_id || self::WPA_HELPER_POST_TYPE !== get_post_type( $post_id ) ) {
			return $condition;
		}

		return true;
	}

	/**
	 * Adds the theme settings related data.
	 *
	 * @param  array $localization_data
	 *
	 * @return array
	 */
	public function add_theme_settings( $localization_data ) {
		$theme_settings = apply_filters( 'wpv_filter_get_theme_settings', array() );

		if ( empty( $theme_settings ) ) {
			return $localization_data;
		}

		$localization_data['themeName'] = $theme_settings['theme_name'];
		$localization_data['themeSlug'] = $theme_settings['theme_slug'];
		$localization_data['collections'] = $theme_settings['collections'];

		return $localization_data;
	}

	/**
	 * Adjusts the frontend post link for WordPress Archive helper posts.
	 * It changes the link to the one of the first assigned loop.
	 * Mostly to be used in the Translation Manager main page and in the Translations page of WPML.
	 *
	 * @param string   $post_link The actual post link.
	 * @param \WP_Post $post      The WP_Post object.
	 *
	 * @return string|null The filtered post link.
	 *
	 * @throws \Exception
	 */
	public function adjust_view_post_type_link( $post_link, $post ) {
		if ( ! $this->is_wpa_helper( $post ) ) {
			return $post_link;
		}

		$wpa = call_user_func( $this->view_base_get_instance, $post->post_parent );

		// $wpa can be null here when the translation WPA Helper post is passed through the "adjust_view_post_type_link".
		// In this case we need to switch to the original WPA Helper post for the rest of the process.
		if ( ! $wpa ) {
			$original_wpa_helper_post_id = apply_filters( 'wpv_filter_get_original_language_ct_post_id_from_translation', $post->ID );

			if ( $original_wpa_helper_post_id <= 0 ) {
				return null;
			}

			$original_wpa_helper_post = get_post( $original_wpa_helper_post_id );

			if (
				! $original_wpa_helper_post ||
				! $original_wpa_helper_post->post_parent
			) {
				return null;
			}

			$wpa = call_user_func( $this->view_base_get_instance, $original_wpa_helper_post->post_parent );

			if ( ! $wpa ) {
				return null;
			}
		}

		if ( $wpa->is_a_wordpress_archive() ) {
			$post_link = null;
			$assigned_loops = $wpa->get_assigned_loops();
			if ( ! empty( $assigned_loops ) ) {
				$loop = array_shift( $assigned_loops );
				while (
					null === $post_link &&
					null !== $loop
				) {
					$link_from_loop = $this->get_post_link_from_loop( $loop );
					$post_link = is_string( $link_from_loop ) ? $link_from_loop : null;
					$loop = array_shift( $assigned_loops );
				}
			}

			$post_is_original_content = apply_filters( 'wpml_is_original_content', true, $post->ID, 'post_' . self::WPA_HELPER_POST_TYPE );
			if (
				$post_link &&
				! $post_is_original_content
			) {
				$from_language = apply_filters( 'wpml_post_language_details', false, $post->ID );
				if (
					false !== $from_language &&
					isset( $from_language['language_code'] )
				) {
					$post_link = apply_filters( 'wpml_permalink', $post_link, $from_language['language_code'] );
				}
			}
		}

		return $post_link;
	}

	/**
	 * Gets the frontend post link for the given loop.
	 *
	 * @param  array $loop The given loop.
	 *
	 * @return mixed|string|null
	 *
	 * @throws \Exception
	 */
	private function get_post_link_from_loop( $loop ) {
		$post_link = null;

		switch ( $loop['loop_type'] ) {
			case 'native':
				$wp_loops = $this->wpv_editor_loop_selection->get_wp_loops_non_static();
				$post_link = $wp_loops[ $loop['slug'] ]['archive_url'];
				break;
			case 'post_type':
				$pt_loops = $this->wpv_editor_loop_selection->get_pt_loops_non_static();
				$post_link = $pt_loops[ $loop['post_type_name'] ]['archive_url'];
				break;
			case 'taxonomy':
				$tax_loops = $this->wpv_editor_loop_selection->get_taxonomy_loops_non_static();
				$taxonomy_slug = $loop['slug'];

				// Get ID of a term that has some posts, if such term exists.
				$terms_with_posts = get_terms(
					$tax_loops[ $taxonomy_slug ],
					array(
						'hide_empty' => 1,
						'number' => 1,
					)
				);

				if (
					! $terms_with_posts instanceof WP_Error &&
					is_array( $terms_with_posts ) &&
					! empty( $terms_with_posts )
				) {
					$terms_with_posts = array_values( $terms_with_posts );
					$term = array_shift( $terms_with_posts );
					$term_link = get_term_link( $term, $taxonomy_slug );
					$post_link = is_string( $term_link ) ? $term_link : null;
				}
				break;
		}

		return $post_link;
	}

	public function maybe_delete_wpa_helper_view_link( $post_view_link, $link_text, $current_document, $element_type, $type ) {
		if ( self::WPA_HELPER_POST_TYPE === $type ) {
			$matches = null;

			preg_match(
				'/<a href="([^"]*)"[^>]*>/',
				$post_view_link,
				$matches
			);

			// It doesn't have content in the href tag.
			if ( '' === $matches[1] ) {
				$post_view_link = null;
			}
		}
		return $post_view_link;
	}
}
